@@ -26,7 +26,6 @@ import json
 import os
 import os.path
 import re
-import shlex
 import shutil
 import subprocess
 import sys
@@ -37,7 +36,11 @@ import click
 from circuitpython_build_tools import build
 from circuitpython_build_tools import target_versions
 
-import pkg_resources
+if sys.version_info < (3, 8):
+    import importlib_metadata
+else:
+    import importlib.metadata as importlib_metadata
+
 
 BLINKA_LIBRARIES = [
     "adafruit-blinka",
@@ -115,6 +118,10 @@ def build_bundle_json(libs, bundle_version, output_filename, package_folder_pref
     Generate a JSON file of all the libraries in libs
     """
     packages = {}
+    # TODO simplify this 2-step process
+    # It mostly exists so that get_bundle_requirements has a way to look up
+    # "pypi name to bundle name" via `package_list[pypi_name]["module_name"]`
+    # otherwise it's just shuffling info around
     for library_path in libs:
         package = {}
         package_info = build.get_package_info(library_path, package_folder_prefix)
@@ -130,15 +137,16 @@ def build_bundle_json(libs, bundle_version, output_filename, package_folder_pref
             packages[module_name] = package
 
     library_submodules = {}
-    for id in packages:
+    for package in packages.values():
         library = {}
-        library["package"] = packages[id]["is_folder"]
-        library["pypi_name"] = packages[id]["pypi_name"]
-        library["version"] = packages[id]["version"]
-        library["repo"] = packages[id]["repo"]
-        library["path"] = packages[id]["path"]
-        library["dependencies"], library["external_dependencies"] = get_bundle_requirements(packages[id]["library_path"], packages)
-        library_submodules[packages[id]["module_name"]] = library
+        library["package"] = package["is_folder"]
+        library["pypi_name"] = package["pypi_name"]
+        library["version"] = package["version"]
+        library["repo"] = package["repo"]
+        library["path"] = package["path"]
+        library["dependencies"], library["external_dependencies"] = get_bundle_requirements(package["library_path"], packages)
+        library_submodules[package["module_name"]] = library
+
     out_file = open(output_filename, "w")
     json.dump(library_submodules, out_file, sort_keys=True)
     out_file.close()
@@ -202,7 +210,7 @@ def build_bundle(libs, bundle_version, output_filename, package_folder_prefix,
     print()
     print("Zipping")
 
-    with zipfile.ZipFile(output_filename, 'w') as bundle:
+    with zipfile.ZipFile(output_filename, 'w', compression=zipfile.ZIP_DEFLATED) as bundle:
         build_metadata = {"build-tools-version": build_tools_version}
         bundle.comment = json.dumps(build_metadata).encode("utf-8")
         if multiple_libs:
@@ -227,6 +235,7 @@ def _find_libraries(current_path, depth):
             subdirectories.extend(_find_libraries(path, depth - 1))
     return subdirectories
 
+all_modules = ["py", "mpy", "example", "json"]
 @click.command()
 @click.option('--filename_prefix', required=True, help="Filename prefix for the output zip files.")
 @click.option('--output_directory', default="bundles", help="Output location for the zip files.")
@@ -234,8 +243,9 @@ def _find_libraries(current_path, depth):
 @click.option('--library_depth', default=0, help="Depth of library folders. This is useful when multiple libraries are bundled together but are initially in separate subfolders.")
 @click.option('--package_folder_prefix', default="adafruit_", help="Prefix string used to determine package folders to bundle.")
 @click.option('--remote_name', default="origin", help="Git remote name to use during building")
-@click.option('--ignore', "-i", multiple=True, type=click.Choice(["py", "mpy", "example", "json"]), help="Bundles to ignore building")
-def build_bundles(filename_prefix, output_directory, library_location, library_depth, package_folder_prefix, remote_name, ignore):
+@click.option('--ignore', "-i", multiple=True, type=click.Choice(all_modules), help="Bundles to ignore building")
+@click.option('--only', "-o", multiple=True, type=click.Choice(all_modules), help="Bundles to build building")
+def build_bundles(filename_prefix, output_directory, library_location, library_depth, package_folder_prefix, remote_name, ignore, only):
     os.makedirs(output_directory, exist_ok=True)
 
     package_folder_prefix = package_folder_prefix.split(", ")
@@ -244,10 +254,10 @@ def build_bundles(filename_prefix, output_directory, library_location, library_d
 
     libs = _find_libraries(os.path.abspath(library_location), library_depth)
 
-    pkg = pkg_resources.get_distribution("circuitpython-build-tools")
-    build_tools_version = "devel"
-    if pkg:
-        build_tools_version = pkg.version
+    try:
+        build_tools_version = importlib_metadata.version("circuitpython-build-tools")
+    except importlib_metadata.PackageNotFoundError:
+        build_tools_version = "devel"
 
     build_tools_fn = "z-build_tools_version-{}.ignore".format(
         build_tools_version)
@@ -255,6 +265,11 @@ def build_bundles(filename_prefix, output_directory, library_location, library_d
     with open(build_tools_fn, "w") as f:
         f.write(build_tools_version)
 
+    if ignore and only:
+        raise SystemExit("Only specify one of --ignore / --only")
+    if only:
+        ignore = set(all_modules) - set(only)
+
     # Build raw source .py bundle
     if "py" not in ignore:
         zip_filename = os.path.join(output_directory,
@@ -267,13 +282,8 @@ def build_bundles(filename_prefix, output_directory, library_location, library_d
     if "mpy" not in ignore:
         os.makedirs("build_deps", exist_ok=True)
         for version in target_versions.VERSIONS:
-            # Use prebuilt mpy-cross on Travis, otherwise build our own.
-            if "TRAVIS" in os.environ:
-                mpy_cross = pkg_resources.resource_filename(
-                    target_versions.__name__, "data/mpy-cross-" + version["name"])
-            else:
-                mpy_cross = "build_deps/mpy-cross-" + version["name"] + (".exe" * (os.name == "nt"))
-                build.mpy_cross(mpy_cross, version["tag"])
+            mpy_cross = "build_deps/mpy-cross-" + version["name"] + (".exe" * (os.name == "nt"))
+            build.mpy_cross(mpy_cross, version["tag"])
             zip_filename = os.path.join(output_directory,
                 filename_prefix + '-{TAG}-mpy-{VERSION}.zip'.format(
                     TAG=version["name"],
